﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;

using Microsoft.SiteAnalyzer.Core;
using Microsoft.SiteAnalyzer.Crawler;

namespace Microsoft.SiteAnalyzer.RandomSampler
{
  class Program
    {
        [STAThread]
        static void Main(string[] args)
        {
           if (args.Length < 3 || args.Length > 10)
            {
                PrintUsage();
                return;
            }

            // parse arguments
            try
            {
                targetUrl = args[0];
                targetHost = new Uri(targetUrl);

                domainConstraint = args[1].ToLower();
                saveDir = args[2];

                if (args.Length > 3)
                {
                    maxSampleNum = Convert.ToInt32(args[3]);
                }
                if (args.Length > 4)
                {
                    maxPerPage = Convert.ToInt32(args[4]);
                }
                if (args.Length > 5)
                {
                    maxThreadNum = Convert.ToInt32(args[5]);
                }
                if (args.Length > 6)
                {
                    maxDepth = Convert.ToInt32(args[6]);
                }
                if (args.Length > 7)
                {
                    sleepInterval = Convert.ToDouble(args[7]);
                }
                if (args.Length > 8)
                {
                    siteEncoder = args[8];
                }
                PageFetcher.SiteEncoder = siteEncoder;
                if (args.Length > 9)
                {
                    allowRedirect = Convert.ToBoolean(args[9]);
                }
                PageFetcher.AllowRedirect = allowRedirect;
            }
            catch (Exception e)
            {
                Console.WriteLine("Error in passing arguments: {0}", e.ToString());
            }

            // prepare output dir and files
            try
            {
                if (!Directory.Exists(saveDir))
                {
                    Directory.CreateDirectory(saveDir);
                }
                if (File.Exists(saveDir + @"\meta.txt"))
                {
                    File.Delete(saveDir + @"\meta.txt");
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("The process failed: {0}", e.ToString());
            }

            // prepare the html parser
            PageInfoParser pageInfoParser = new PageInfoParser();
            pageInfoParser.DomainBase = domainConstraint;

            bool followOutlink = true;

            // setup the crawlers
            c = new CrawlerPool(
                targetHost, //URL of the portal page
                domainConstraint, //domain constraint
                maxSampleNum,  //expected number
                false, //usePriority
                pageInfoParser, //DOM Parser
                maxThreadNum, 
                maxDepth, 
                sleepInterval,
                false, //onlyCrawlText
                3, //maxRetry
                allowRedirect,
                followOutlink); //follow outlinks 
            c.OnPageCrawled += new CrawlerPool.PageCrawledDelegation(c_OnPageCrawled);
            c.OnPageFailed += new CrawlerPool.PageFailedDelegation(c_OnPageFailed);
            c.OnCrawlComplete += new CrawlerPool.CrawlCompleteDelegation(c_OnCrawlComplete);
            c.OnDiscoverUrls += new CrawlerPool.DiscoverUrlDelegation(c_OnDiscoverUrl);

            // let's begin
            List<string> targetUrlList = new List<string>();
            targetUrlList.Add(targetUrl);
            c.Start(targetUrlList.ToArray(), true);
        }

        static void c_OnPageCrawled(int threadId, string url, int layoutId, byte[] content, bool isText, int depth, out bool crawlAnchors)
        {
            crawlAnchors = true;

            lock (dumpLock)
            {
                using (FileStream fWriter = new FileStream(saveDir + @"\" + c.CurCrawledNum + @".htm", FileMode.Create))
                {
                    string baseUrl = string.Format("<base href=\"{0}\"/>\n", url);
                    foreach (char ch in baseUrl)
                    {
                        fWriter.WriteByte((byte)ch);
                    }

                    fWriter.Write(content, 0, content.Length);
                    fWriter.Close();
                }

                using (StreamWriter metaWriter = new StreamWriter(saveDir + @"\meta.txt", true))
                {
                    metaWriter.WriteLine("" + c.CurCrawledNum + "\t" + url);
                }

                c.CurCrawledNum++;
            }

            System.Console.WriteLine("[SUCCEED] : (ThreadId:" + threadId + ") " + c.CurCrawledNum + "/" + c.GetQueueVolumn() + " [" + depth + "] " + url);
        }

        static void c_OnPageFailed(string url, int retryTimes, int depth, out bool continueRetry)
        {
            continueRetry = true;

            System.Console.WriteLine("[FAILED] (Retry:" + retryTimes + ") : [" + depth + "] " + url);
        }

        static void c_OnCrawlComplete()
        {
            System.Console.WriteLine("[Finished] : {0} pages crawled, {1} pages failed.", c.CurCrawledNum, c.CurFailedNum);
        }

        static List<PageInfo.OutLink> c_OnDiscoverUrl(PageInfo page_info, int depth)
        {
            List<PageInfo.OutLink> outlinks = new List<PageInfo.OutLink>();
            
            if (page_info.OutLinks == null)
                return outlinks;

            for (int i = 0; i < page_info.OutLinks.Length; i++)
            {
                try
                {
                    Uri absUrl = new Uri(new Uri(page_info.Url), page_info.OutLinks[i].Url);
                    string normurl = absUrl.AbsoluteUri;

                     if (normurl.IndexOf(domainConstraint) >= 0)
                    {
                        PageInfo.OutLink link = new PageInfo.OutLink();
                        link.Url = normurl;
                        //link.TagPath = page_info.OutLiks[i].TagPath;
                        outlinks.Add(link);
                    }
                }
                catch (UriFormatException)
                {
                    Console.WriteLine("Error Uri : " + page_info.OutLinks[i].Url);
                }
            }

            return PageSampler.GetFollowingUrls(outlinks, depth, maxPerPage);
        }

        static void PrintUsage()
        {
            Console.WriteLine("SiteAnalyzer Utility - Random Sampler");
            Console.WriteLine("====================================================");
            Console.WriteLine("Command line arguments : >Microsoft.SiteAnalyzer.RandomSampler [0] [1] [2] (4) (5) (6) (7) (8) (9)");
            Console.WriteLine("[0] : The URL of the portal page of the target website.");
            Console.WriteLine("[1] : The domain constraint, only sampling pages in this domain.");
            Console.WriteLine("[2] : The target directory to save sampled html pages.");
            Console.WriteLine("(3) : The expected number of total crawled pages. default is 5000");
            Console.WriteLine("(4) : The maximum number of out-links followed on each page. default is 20");
            Console.WriteLine("(5) : The maximum number of download threads. default is 5");
            Console.WriteLine("(6) : The maximum download depth. Default is -1, means no depth limitation");
            Console.WriteLine("(7) : The sleep interval between two crawling. Default is 0.05 second");
            Console.WriteLine("(8) : The encoder of this web site (auto/unicode/utf-8/gb2312/...)");
            Console.WriteLine("(9): Allow URL redirect? (true/false) default is false");
            Console.WriteLine("====================================================");
        }

        private static string targetUrl;
        private static Uri targetHost;
        private static string domainConstraint;
        private static string saveDir;
        private static int maxSampleNum = 5000;
        private static int maxPerPage = 20;
        private static int maxThreadNum = 5;
        private static int maxDepth = -1;
        private static double sleepInterval = 0.05;
        private static string siteEncoder = "auto";
        private static bool allowRedirect = false;

        private static CrawlerPool c = null;
        private static object dumpLock = new object();
    }
 }
